//  *****************************************************************************
//   						main.c
// 
//              Demonstracny program pre dosku KTK ARM7
//
//              Zobrazovanie cislel 0-99 na dvoch 7-segmentovkach pomocou multiplexu
//
//  *****************************************************************************

// Hlavickove subory
#include "board.h"

// Definicia konstant pre zobrazovanie znakov na 7-segmentovke
char znaky[16]={0x3f,    // znak '0'
                0x06,    // znak '1'
                0x5b,    // znak '2'
                0x4f,    // znak '3'
                0x66,    // znak '4'
                0x6d,    // znak '5'
                0x7d,    // znak '6'
                0x07,    // znak '7'
                0x7f,    // znak '8'
                0x6f,    // znak '9'
                0x77,    // znak 'A'
                0x7c,    // znak 'b'
                0x58,    // znak 'c'
                0x5e,    // znak 'd'
                0x79,    // znak 'E'
                0x71};   // znak 'F'

char znak1, znak2;  // znaky, ktore sa maju zobrazovat na 7-segmentovkach

#define SEG1     1 << 8             // anoda prvej sedemsegmentovky - PA8 (aktivacia nastavenim na 0)
#define SEG2     1 << 9             // anoda druhej sedemsegmentovky - PA9 (aktivacia nastavenim na 0)
#define PIT_INTERRUPT_LEVEL 0       // 0 - najnizsia priorita prerusenia


//  *******************************************************
//  Kratke cakanie (cca 450 us)
//  *******************************************************
void cakaj(void)
{ unsigned int cas;
 
  for(cas=0; cas < MCK/20000; cas++);
}

//  *******************************************************
//  Obsluha prerusenia od periodickeho casovaca PIT
//  *******************************************************
void PIT_prerusenie(void)
{
  // vymazanie ziadosti o prerusenie
  AT91C_BASE_PITC->PITC_PIVR;

  AT91C_BASE_PIOA->PIO_SODR = 0xff;	      // zhasnutie znaku
  // prepnutie zobrazovanej 7-segmentovky
  if (AT91C_BASE_PIOA->PIO_PDSR & SEG2)       // ak je SEG2 vypnuta
  { AT91C_BASE_PIOA->PIO_SODR = SEG1;	      // vypnutie prvej 7-seg.
    AT91C_BASE_PIOA->PIO_CODR = znak2;	      // nastavenie druheho znaku
    AT91C_BASE_PIOA->PIO_CODR = SEG2;	      // zapnutie druhej 7-seg.
  }
  else
  { AT91C_BASE_PIOA->PIO_SODR = SEG2;	      // vypnutie druhej 7-seg.
    AT91C_BASE_PIOA->PIO_CODR = znak1;	      // nastavenie prveho znaku
    AT91C_BASE_PIOA->PIO_CODR = SEG1;	      // zapnutie prvej 7-seg.
  }
}

//  *******************************************************
//                Hlavny program
//  *******************************************************
int main(void) 
{ int index;

  //******************* Uvodna inicializacia *******************
  // Povolenie Resetu - minimalna dlzka 960 us
  AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);
	
  //******************* Inicializacia portu A ******************

  // Povolenie hodin pre PIOA
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_PIOA; 
	
  // konfiguracia jednotlivych pinov
  // vstupne piny
  AT91C_BASE_PIOA->PIO_ODR = S2;
  // vystupne piny
  AT91C_BASE_PIOA->PIO_OER = 0xff | SEG1 | SEG2;       // piny A0-A7,SEG1,SEG2 su vystupne
  // povolenie Pull-Up rezistoru
  AT91C_BASE_PIOA->PIO_PPUER = S2;
  // povolenie vsetkych nakonfigurovanych pinov
  AT91C_BASE_PIOA->PIO_PER = S2 | 0xff | SEG1 | SEG2;

  // zhasnutie 7-segmentoviek
  AT91C_BASE_PIOA->PIO_SODR = SEG1 | SEG2;
  
  // pociatocne nastavenie zobrazovanych znakov
  znak1 = znaky[0];   // '0'
  znak2 = 0;          // nic sa nezobrazuje

  //******************* Nastavenie prerusenia od PIT ******************
  // Povolenie hodin pre PIT
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_SYS; 
  // nastavenie periody casovaca na 0.005 s (200 Hz)
  // do PIT vstupuje frekvencia MCK/16
  AT91C_BASE_PITC->PITC_PIMR = (int)((MCK/16)*0.005)-1   // perioda prerusenia
                               | AT91C_PITC_PITEN        // povolenie casovaca
                               | AT91C_PITC_PITIEN;      // povolenie prerusenia

  // nastavenie obsluzneho programu
  AT91C_BASE_AIC->AIC_SVR[AT91C_ID_SYS] = (unsigned int) PIT_prerusenie;
  // zdroj prerusenia a priorita
  AT91C_BASE_AIC->AIC_SMR[AT91C_ID_SYS] = PIT_INTERRUPT_LEVEL;
  // zrusenie prerusenia (ak nejake nahodou bolo)
  AT91C_BASE_AIC->AIC_ICCR = 0x1 << AT91C_ID_SYS;
  // povolenie prerusenia od PIT
  AT91C_BASE_AIC->AIC_IECR = 0x1 << AT91C_ID_SYS;

  //******************* Hlavna cast programu *******************
  index=0;
  // nekonecna slucka
  while(1)
  { // cakanie na stlacenie S2  
    while(AT91C_BASE_PIOA->PIO_PDSR & S2)     // kontrola, ci je S2 v log.1
      cakaj();                                // cakanie kvoli potlaceniu zakmitov
    // cakanie na pustenie S2
    while(!(AT91C_BASE_PIOA->PIO_PDSR & S2))  // kontrola, ci je S2 v log.0
      cakaj();                                // cakanie kvoli potlaceniu zakmitov

    // zabezpecenie cyklenia od 0 do 99
    index++;
    if (index > 99)
      index=0;

    // priprava znakov na zobrazenie
    if (index<10)
    { znak1 = znaky[index];           // jednomiestne cislo
      znak2 = 0;
    }
    else
    { znak1 = znaky[index%10];        // dvojmiestne cislo
      znak2 = znaky[index/10];
    }
  }
}
